#' Estimates the optimal number of clusters using the Slope criterion when silhouette statistics already exists.
#' The optimal number of clusters will be verified in the range {2,..., maxClust}.
#'
#' @param silStats Silhouette statistics.
#' @param p Slope adjust parameter.
#' @param maxClust The max number of clusters to be tried.
#'
#' @return the optimal number of clusters.
#'
#' @export
#'
optimalSlope <- function(silStats, p = 1, maxClust){
  slope = array(NA, maxClust - 2)
  for (j in seq(maxClust - 2)){
      slope[j] = - (silStats[j+1] - silStats[j]) * (silStats[j]^p)
  }
  slopeIndice = which(slope == max(slope))[1]
  return(slopeIndice + 1)
}

#' Estimates the optimal number of clusters using either Slope or Silhouette criterion. The optimal number of clusters will be
#' verified in the range {2,..., maxClust}.
#'
#' @param dataDist A matrix with n subjects. Each subject has the size of NxN and represents the distances between the elements of the sample.
#' @param p Slope adjust parameter.
#' @param maxClust The max number of clusters to be tried.
#' @param clusteringFunction The clustering function that Slope should use.
#' @param criterion The criterion that will be used for estimating the number of clusters. The options are "slope" or "silhouette". If not defined, "slope" will be used.
#'
#' @return the optimal number of clusters.

#' @export
#'
nClust <- function(dataDist, p = 1, maxClust = 20, clusteringFunction, criterion = c("slope", "silhouette")){

  #Slope is the default criterion for estimating the number of cluster
  criterion <- match.arg(criterion)

  dataMean <- colMeans(dataDist)
  N <- dim(dataDist)[2]

  clusterList <- array(NA, c(maxClust - 1, N))
  silStats <- array(NA, maxClust - 1)

  for(i in seq(2, maxClust)){
    clusters = clusteringFunction(1 - dataMean, i)
    clusterList[i - 1, ] = clusters
    S = silhouette(clusters, dmatrix = dataMean)[ , 3]
    silStats[i - 1] = mean(S)
  }

  if(criterion == "slope"){
    return(optimalSlope(silStats, p, maxClust))
  }else{
    return(optimalSilhouette(silStats))
  }
}

#' Estimates the optimal number of clusters using the Silhouette criterion when silhouette statistics already exists.
#'
#' @param silStats Silhouette statistics for each possible tested number of clusters.
#'
#' @return the optimal number of clusters.

#' @export
#'
optimalSilhouette <- function(silStats){
  return(which.max(silStats) + 1)
}
